@@ -60,11 +60,11 @@ int pocl_cache_write_program_source(char *program_cl_path,
 
 int pocl_cache_write_kernel_objfile (char *objfile_path,
                                      const char *objfile_content,
-                                     size_t objfile_size);
+                                     uint64_t objfile_size);
 
 int pocl_cache_write_spirv (char *spirv_path,
                             const char *spirv_content,
-                            size_t file_size);
+                            uint64_t file_size);
 
 int pocl_cache_update_program_last_access(cl_program program,
                                           unsigned device_i);
@@ -91,7 +91,7 @@ void pocl_cache_kernel_cachedir_path (char* kernel_cachedir_path,
                                       cl_program program,
                                       unsigned device_i,
                                       cl_kernel kernel,
-                                      char* append_str,
+                                      const char* append_str,
                                       size_t local_x,
                                       size_t local_y,
                                       size_t local_z);
@@ -113,7 +113,7 @@ void pocl_cache_program_path(char*        path,
                              unsigned     device_i);
 
 void pocl_cache_kernel_cachedir(char* kernel_cachedir_path, cl_program   program,
-                                unsigned device_i, cl_kernel kernel);
+                                unsigned device_i, const char* kernel_name);
 
 // these two required by llvm API
 
