@@ -25,56 +25,22 @@
 #include <hwloc.h>
 #include <stdlib.h>
 
+#if HWLOC_API_VERSION >= 0x00020000
+#define HWLOC_API_2
+#else
+#undef HWLOC_API_2
+#endif
+
 #include "pocl_topology.h"
 
-#if defined(__x86_64__) || defined(__i386__)
-
-enum VendorSignatures
-{
-  SIG_INTEL = 0x756e6547 /* Genu */,
-  SIG_AMD = 0x68747541 /* Auth */
-};
-
-/// getX86CpuIDAndInfo - Execute the specified cpuid and return the 4 values in
-/// the specified arguments.  If we can't run cpuid on the host, return true.
-static int
-getX86CpuIDAndInfo (unsigned value, unsigned *rEAX, unsigned *rEBX,
-                    unsigned *rECX, unsigned *rEDX)
-{
-#if defined(__GNUC__) || defined(__clang__)
-#if defined(__x86_64__)
-  // gcc doesn't know cpuid would clobber ebx/rbx. Preserve it manually.
-  __asm__("movq\t%%rbx, %%rsi\n\t"
-          "cpuid\n\t"
-          "xchgq\t%%rbx, %%rsi\n\t"
-          : "=a"(*rEAX), "=S"(*rEBX), "=c"(*rECX), "=d"(*rEDX)
-          : "a"(value));
-  return 0;
-#elif defined(__i386__)
-  __asm__("movl\t%%ebx, %%esi\n\t"
-          "cpuid\n\t"
-          "xchgl\t%%ebx, %%esi\n\t"
-          : "=a"(*rEAX), "=S"(*rEBX), "=c"(*rECX), "=d"(*rEDX)
-          : "a"(value));
-  return 0;
-#else
-  return 1;
-#endif
-#elif defined(_MSC_VER)
-  // The MSVC intrinsic is portable across x86 and x64.
-  int registers[4];
-  __cpuid (registers, value);
-  *rEAX = registers[0];
-  *rEBX = registers[1];
-  *rECX = registers[2];
-  *rEDX = registers[3];
-  return 0;
-#else
-  return 1;
-#endif
-}
-
-#endif
+/*
+ * Sets up:
+ *  max_compute_units
+ *  global_mem_size
+ *  global_mem_cache_type
+ *  global_mem_cacheline_size
+ *  global_mem_cache_size
+ */
 
 int
 pocl_topology_detect_device_info(cl_device_id device)
@@ -82,6 +48,16 @@ pocl_topology_detect_device_info(cl_device_id device)
   hwloc_topology_t pocl_topology;
   int ret = 0;
 
+#ifdef HWLOC_API_2
+  if (hwloc_get_api_version () < 0x20000)
+    POCL_MSG_ERR ("pocl was compiled against libhwloc 2.x but is"
+                  "actually running against libhwloc 1.x \n");
+#else
+  if (hwloc_get_api_version () >= 0x20000)
+    POCL_MSG_ERR ("pocl was compiled against libhwloc 1.x but is"
+                  "actually running against libhwloc 2.x \n");
+#endif
+
   /*
    * hwloc's OpenCL backend causes problems at the initialization stage
    * because it reloads libpocl.so via the ICD loader.
@@ -101,6 +77,15 @@ pocl_topology_detect_device_info(cl_device_id device)
     return ret;
   }
 
+#ifdef HWLOC_API_2
+  hwloc_topology_set_io_types_filter(pocl_topology, HWLOC_TYPE_FILTER_KEEP_NONE);
+  hwloc_topology_set_type_filter (pocl_topology, HWLOC_OBJ_SYSTEM, HWLOC_TYPE_FILTER_KEEP_NONE);
+  hwloc_topology_set_type_filter (pocl_topology, HWLOC_OBJ_GROUP, HWLOC_TYPE_FILTER_KEEP_NONE);
+  hwloc_topology_set_type_filter (pocl_topology, HWLOC_OBJ_BRIDGE, HWLOC_TYPE_FILTER_KEEP_NONE);
+  hwloc_topology_set_type_filter (pocl_topology, HWLOC_OBJ_MISC, HWLOC_TYPE_FILTER_KEEP_NONE);
+  hwloc_topology_set_type_filter (pocl_topology, HWLOC_OBJ_PCI_DEVICE, HWLOC_TYPE_FILTER_KEEP_NONE);
+  hwloc_topology_set_type_filter (pocl_topology, HWLOC_OBJ_OS_DEVICE, HWLOC_TYPE_FILTER_KEEP_NONE);
+#else
   hwloc_topology_ignore_type (pocl_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_IO);
   hwloc_topology_ignore_type (pocl_topology, HWLOC_OBJ_SYSTEM);
   hwloc_topology_ignore_type (pocl_topology, HWLOC_OBJ_GROUP);
@@ -108,6 +93,7 @@ pocl_topology_detect_device_info(cl_device_id device)
   hwloc_topology_ignore_type (pocl_topology, HWLOC_OBJ_MISC);
   hwloc_topology_ignore_type (pocl_topology, HWLOC_OBJ_PCI_DEVICE);
   hwloc_topology_ignore_type (pocl_topology, HWLOC_OBJ_OS_DEVICE);
+#endif
 
   ret = hwloc_topology_load (pocl_topology);
   if (ret == -1)
@@ -116,36 +102,19 @@ pocl_topology_detect_device_info(cl_device_id device)
     goto exit_destroy;
   }
 
+#ifdef HWLOC_API_2
+  device->global_mem_size =
+      hwloc_get_root_obj(pocl_topology)->total_memory;
+#else
   device->global_mem_size =
       hwloc_get_root_obj(pocl_topology)->memory.total_memory;
+#endif
 
   // Try to get the number of CPU cores from topology
   int depth = hwloc_get_type_depth(pocl_topology, HWLOC_OBJ_PU);
   if(depth != HWLOC_TYPE_DEPTH_UNKNOWN)
     device->max_compute_units = hwloc_get_nbobjs_by_depth(pocl_topology, depth);
 
-#if defined(__GNUC__) || defined(__clang__) || defined(_MSC_VER)
-#if defined(__x86_64__) || defined(__i386__)
-  unsigned Vendor, EAX, ECX, EDX;
-  if (getX86CpuIDAndInfo (0, &EAX, &Vendor, &ECX, &EDX))
-    device->vendor_id = 0x0086;
-  else
-    {
-      if (Vendor == SIG_INTEL)
-        device->vendor_id = 0x8086;
-      else if (Vendor == SIG_AMD)
-        device->vendor_id = 0x1022;
-      else
-        /* unknown x86 */
-        device->vendor_id = 0x0086;
-    }
-#else
-  device->vendor_id = 0x0000;
-#endif
-#else
-  device->vendor_id = 0x0000;
-#endif
-
   /* Find information about global memory cache by looking at the first
    * cache covering the first PU */
   do {
