@@ -1,7 +1,7 @@
 /* OpenCL runtime library: compile_and_link_program()
 
    Copyright (c) 2011-2013 Universidad Rey Juan Carlos,
-                 2011-2014 Pekka Jääskeläinen / Tampere Univ. of Technology
+                 2011-2018 Pekka Jääskeläinen
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
@@ -114,21 +114,24 @@ static const char cl_parameters_not_yet_supported_by_clang[] =
   }
 
 #ifdef OCS_AVAILABLE
+/* if 'only_spmd_devices' is set, prebuild the kernel binaries only for the
+ * SPMD devices in the context, otherwise build binaries for all devices.
+ * The former is useful for clBuildProgram(), the latter for
+ * clGetProgramInfo(CL_PROGRAM_BINARIES). */
 cl_int
-program_compile_dynamic_wg_binaries(cl_program program)
+program_compile_dynamic_wg_binaries (cl_program program, int only_spmd_devices)
 {
   unsigned i, device_i;
-  cl_int errcode = CL_SUCCESS;
   _cl_command_node cmd;
 
-  assert(program->num_kernels);
   assert(program->build_status == CL_BUILD_SUCCESS);
+  if (program->num_kernels == 0)
+    return CL_SUCCESS;
 
   memset(&cmd, 0, sizeof(_cl_command_node));
   cmd.type = CL_COMMAND_NDRANGE_KERNEL;
-  char cachedir[POCL_FILENAME_LENGTH];
-  cmd.command.run.tmp_dir = cachedir;
-  POCL_LOCK_OBJ(program);
+
+  POCL_LOCK_OBJ (program);
 
   /* Build the dynamic WG sized parallel.bc and device specific code,
      for each kernel & device combo.  */
@@ -140,33 +143,49 @@ program_compile_dynamic_wg_binaries(cl_program program)
       if (program->pocl_binaries[device_i] || (!program->binaries[device_i]))
         continue;
 
+      if (only_spmd_devices && (device->spmd == CL_FALSE))
+        continue;
+
       cmd.device = device;
+      cmd.command.run.device_i = device_i;
+
+      struct _cl_kernel fake_k;
+      memset (&fake_k, 0, sizeof (fake_k));
+      fake_k.context = program->context;
+      fake_k.program = program;
+      fake_k.next = NULL;
+      cl_kernel kernel = &fake_k;
 
       for (i=0; i < program->num_kernels; i++)
         {
-          cl_kernel kernel = program->default_kernels[i];
+          fake_k.meta = &program->kernel_meta[i];
+          fake_k.name = fake_k.meta->name;
+          cmd.command.run.hash = fake_k.meta->build_hash[device_i];
+
           size_t local_x = 0, local_y = 0, local_z = 0;
-          if (kernel->reqd_wg_size != NULL &&
-              kernel->reqd_wg_size[0] > 0 &&
-              kernel->reqd_wg_size[1] > 0 &&
-              kernel->reqd_wg_size[2] > 0)
+
+          if (kernel->meta->reqd_wg_size[0] > 0
+              && kernel->meta->reqd_wg_size[1] > 0
+              && kernel->meta->reqd_wg_size[2] > 0)
             {
-              local_x = kernel->reqd_wg_size[0];
-              local_y = kernel->reqd_wg_size[1];
-              local_z = kernel->reqd_wg_size[2];
+              local_x = kernel->meta->reqd_wg_size[0];
+              local_y = kernel->meta->reqd_wg_size[1];
+              local_z = kernel->meta->reqd_wg_size[2];
             }
+
           cmd.command.run.local_x = local_x;
           cmd.command.run.local_y = local_y;
           cmd.command.run.local_z = local_z;
+
           cmd.command.run.kernel = kernel;
-          pocl_cache_kernel_cachedir_path (cachedir, program, device_i, kernel,
-                                           "", local_x, local_y, local_z);
+
           device->ops->compile_kernel (&cmd, kernel, device);
         }
     }
 
-  POCL_UNLOCK_OBJ(program);
-  return errcode;
+  POCL_UNLOCK_OBJ (program);
+
+  return CL_SUCCESS;
 }
 
 #endif
@@ -202,7 +221,8 @@ process_options (const char *options, char *modded_options, char *link_options,
 
   size_t i = 1; /* terminating char */
   size_t needed = 0;
-  char *temp_options = strdup (options);
+  char *temp_options = (char*) malloc (strlen (options) + 1);
+  strcpy (temp_options, options);
 
   token = strtok_r (temp_options, " ", &saveptr);
   while (token != NULL)
@@ -273,7 +293,8 @@ process_options (const char *options, char *modded_options, char *link_options,
       else if (memcmp (token, "-g", 2) == 0)
         {
 #ifndef LLVM_OLDER_THAN_3_8
-          token = "-debug-info-kind=line-tables-only";
+          token = "-dwarf-column-info -debug-info-kind=limited " \
+	    "-dwarf-version=4 -debugger-tuning=gdb";
 #endif
         }
       else if (memcmp (token, "-D", 2) == 0 || memcmp (token, "-I", 2) == 0)
@@ -383,6 +404,33 @@ ERROR:
   return error;
 }
 
+/*****************************************************************************/
+
+static void
+free_meta (cl_program program)
+{
+  size_t i;
+  unsigned j;
+
+  if (program->num_kernels)
+    {
+      for (i = 0; i < program->num_kernels; i++)
+        {
+          pocl_kernel_metadata_t *meta = &program->kernel_meta[i];
+          POCL_MEM_FREE (meta->attributes);
+          POCL_MEM_FREE (meta->name);
+          POCL_MEM_FREE (meta->arg_info);
+          for (j = 0; j < program->num_devices; ++j)
+            if (meta->data[j] != NULL)
+              meta->data[j] = NULL; // TODO free data in driver callback
+          POCL_MEM_FREE (meta->data);
+          POCL_MEM_FREE (meta->local_sizes);
+          POCL_MEM_FREE (meta->build_hash);
+        }
+      POCL_MEM_FREE (program->kernel_meta);
+    }
+}
+
 static void
 clean_program_on_rebuild (cl_program program)
 {
@@ -391,29 +439,17 @@ clean_program_on_rebuild (cl_program program)
   size_t i;
   if ((program->build_status != CL_BUILD_NONE) || program->num_kernels > 0)
     {
-      cl_kernel k;
-      for (k = program->kernels; k != NULL; k = k->next)
-        {
-          k->program = NULL;
-          --program->pocl_refcount;
-        }
-      program->kernels = NULL;
-      if (program->num_kernels)
-        {
-          program->operating_on_default_kernels = 1;
-          for (i = 0; i < program->num_kernels; i++)
-            {
-              if (program->kernel_names)
-                POCL_MEM_FREE (program->kernel_names[i]);
-              if (program->default_kernels && program->default_kernels[i])
-                POname (clReleaseKernel) (program->default_kernels[i]);
-            }
-          POCL_MEM_FREE (program->kernel_names);
-          POCL_MEM_FREE (program->default_kernels);
-          program->operating_on_default_kernels = 0;
-        }
+      /* Spec says:
+         CL_INVALID_OPERATION if there are kernel objects attached to program.
+         ...and we check for that earlier.
+       */
+      assert (program->kernels == NULL);
+
+      free_meta (program);
+
       program->num_kernels = 0;
       program->build_status = CL_BUILD_NONE;
+
       for (i = 0; i < program->num_devices; ++i)
         {
           POCL_MEM_FREE (program->build_log[i]);
@@ -422,29 +458,31 @@ clean_program_on_rebuild (cl_program program)
             {
               POCL_MEM_FREE (program->binaries[i]);
               program->binary_sizes[i] = 0;
-              POCL_MEM_FREE (program->llvm_irs[i]);
+#ifdef OCS_AVAILABLE
+              if (program->llvm_irs[i])
+                pocl_free_llvm_irs (program, i);
+#endif
+              POCL_MEM_FREE (program->pocl_binaries[i]);
+              program->pocl_binary_sizes[i] = 0;
             }
         }
       program->main_build_log[0] = 0;
     }
 }
 
+
 cl_int
 compile_and_link_program(int compile_program,
                          int link_program,
                          cl_program program,
-
                          cl_uint num_devices,
                          const cl_device_id *device_list,
                          const char *options,
-
                          cl_uint num_input_headers,
                          const cl_program *input_headers,
                          const char **header_include_names,
-
                          cl_uint num_input_programs,
                          const cl_program *input_programs,
-
                          void (CL_CALLBACK *pfn_notify) (cl_program program,
                                                          void *user_data),
                          void *user_data)
@@ -461,8 +499,12 @@ compile_and_link_program(int compile_program,
   char *binary = NULL;
   unsigned device_i = 0, actually_built = 0;
   size_t i, j;
+  char *temp_options = NULL;
+  const char *extra_build_options =
+    pocl_get_string_option ("POCL_EXTRA_BUILD_FLAGS", NULL);
   int build_error_code
       = (link_program ? CL_BUILD_PROGRAM_FAILURE : CL_COMPILE_PROGRAM_FAILURE);
+
   POCL_GOTO_LABEL_COND (PFN_NOTIFY, (program == NULL), CL_INVALID_PROGRAM);
 
   POCL_GOTO_LABEL_COND (PFN_NOTIFY, (num_devices > 0 && device_list == NULL),
@@ -494,12 +536,29 @@ compile_and_link_program(int compile_program,
   /* TODO this should be somehow utilized at linking */
   POCL_MEM_FREE (program->compiler_options);
 
-  if (options)
+  if (extra_build_options)
     {
-      i = strlen (options);
+      size_t len = (options != NULL) ? strlen (options) : 0;
+      len += strlen (extra_build_options) + 2;
+      temp_options = (char *)malloc (len);
+
+      temp_options[0] = 0;
+      if (options != NULL)
+        {
+          strcpy (temp_options, options);
+          strcat (temp_options, " ");
+        }
+      strcat (temp_options, extra_build_options);
+    }
+  else
+    temp_options = (char*) options;
+
+  if (temp_options)
+    {
+      i = strlen (temp_options);
       size_t size = i + 512; /* add some space for pocl-added options */
       program->compiler_options = (char *)malloc (size);
-      errcode = process_options (options, program->compiler_options,
+      errcode = process_options (temp_options, program->compiler_options,
                                  link_options, program, compile_program,
                                  link_program, &create_library, &flush_denorms,
                                  &requires_cr_sqrt_div, &spir_build, size);
@@ -507,7 +566,7 @@ compile_and_link_program(int compile_program,
         goto ERROR_CLEAN_OPTIONS;
     }
 
-  POCL_MSG_PRINT_INFO ("building program with options %s\n",
+  POCL_MSG_PRINT_LLVM ("building program with options %s\n",
                        program->compiler_options);
 
 
@@ -563,21 +622,27 @@ compile_and_link_program(int compile_program,
       /* clCreateProgramWithSource */
       if (program->source)
         {
-          POCL_MSG_PRINT_INFO("building from sources for device %d\n", device_i);
 #ifdef OCS_AVAILABLE
-          error = pocl_llvm_build_program(
-              program, device_i, program->compiler_options, program_bc_path,
-              num_input_headers, input_headers, header_include_names,
-              (create_library ? 0 : link_program));
-          POCL_GOTO_ERROR_ON ((error != 0), build_error_code,
-                              "pocl_llvm_build_program() failed\n");
-#else
-          APPEND_TO_MAIN_BUILD_LOG (
-              "Cannot build a program from sources with pocl "
-              "that does not have online compiler support\n");
-          POCL_GOTO_ERROR_ON(1, CL_COMPILER_NOT_AVAILABLE,
-                             "%s", program->main_build_log);
+          if (device->compiler_available == CL_TRUE)
+            {
+              POCL_MSG_PRINT_INFO ("building from sources for device %d\n",
+                                   device_i);
+              error = pocl_llvm_build_program (
+                  program, device_i, program->compiler_options,
+                  program_bc_path, num_input_headers, input_headers,
+                  header_include_names, (create_library ? 0 : link_program));
+              POCL_GOTO_ERROR_ON ((error != 0), build_error_code,
+                                  "pocl_llvm_build_program() failed\n");
+            }
+          else
 #endif
+            {
+              APPEND_TO_MAIN_BUILD_LOG (
+                  "Cannot build a program from sources with pocl "
+                  "that does not have online compiler support\n");
+              POCL_GOTO_ERROR_ON (1, CL_COMPILER_NOT_AVAILABLE, "%s",
+                                  program->main_build_log);
+            }
         }
       /* clCreateProgramWithBinaries */
       else if (program->binaries[device_i]
@@ -711,7 +776,6 @@ compile_and_link_program(int compile_program,
        * cache directory. Will be useful for cache pruning script
        * that flushes old directories based on LRU */
       pocl_cache_update_program_last_access(program, device_i);
-
 #endif
 
     }
@@ -735,91 +799,79 @@ compile_and_link_program(int compile_program,
     program->binary_type = CL_PROGRAM_BINARY_TYPE_COMPILED_OBJECT;
 
   assert(program->num_kernels == 0);
-  for (i=0; i < program->num_devices; i++)
+
+  /* get non-device-specific kernel metadata. We can stop after finding
+   * the first method that works.*/
+  for (device_i = 0; device_i < program->num_devices; device_i++)
     {
 #ifdef OCS_AVAILABLE
-      if (program->binaries[i])
+      if (program->binaries[device_i])
         {
-          program->num_kernels = pocl_llvm_get_kernel_count(program);
+          program->num_kernels
+              = pocl_llvm_get_kernel_count (program, device_i);
           if (program->num_kernels)
             {
-              program->kernel_names = calloc(program->num_kernels, sizeof(char*));
-              pocl_llvm_get_kernel_names(program,
-                                         program->kernel_names,
-                                         program->num_kernels);
+              program->kernel_meta = calloc (program->num_kernels,
+                                             sizeof (pocl_kernel_metadata_t));
+              pocl_llvm_get_kernels_metadata (program, device_i);
             }
           break;
         }
 #endif
-      if (program->pocl_binaries[i])
+      if (program->pocl_binaries[device_i])
         {
-          program->num_kernels =
-              pocl_binary_get_kernel_count(program->pocl_binaries[i]);
+          program->num_kernels
+              = pocl_binary_get_kernel_count (program, device_i);
           if (program->num_kernels)
             {
-              program->kernel_names = calloc(program->num_kernels, sizeof(char*));
-              pocl_binary_get_kernel_names(program->pocl_binaries[i],
-                                           program->kernel_names,
-                                           program->num_kernels);
+              program->kernel_meta = calloc (program->num_kernels,
+                                             sizeof (pocl_kernel_metadata_t));
+              pocl_binary_get_kernels_metadata (program, device_i);
             }
           break;
         }
     }
-  POCL_GOTO_ERROR_ON((i >= program->num_devices),
-                     CL_INVALID_BINARY,
-                     "Could not set kernel number / names from the binary\n");
 
-  /* Set up all program kernels.  */
-  assert (program->default_kernels == NULL);
-  program->operating_on_default_kernels = 1;
-  if (program->num_kernels > 0)
-    program->default_kernels
-        = calloc (program->num_kernels, sizeof (cl_kernel));
+  POCL_GOTO_ERROR_ON ((device_i >= program->num_devices), CL_INVALID_BINARY,
+                      "Could find kernel metadata in the built program\n");
 
-  for (i=0; i < program->num_kernels; i++)
+  /* calculate device-specific kernel hashes. */
+  for (j = 0; j < program->num_kernels; ++j)
     {
-      program->default_kernels[i] =
-          POname (clCreateKernel) (program, program->kernel_names[i], &errcode);
-      POCL_GOTO_ERROR_ON ((errcode != CL_SUCCESS), build_error_code,
-                          "Failed to create default kernels\n");
+      program->kernel_meta[j].build_hash
+          = calloc (program->num_devices, sizeof (pocl_kernel_hash_t));
+
+      for (device_i = 0; device_i < program->num_devices; device_i++)
+        {
+          pocl_calculate_kernel_hash (program, j, device_i);
+        }
     }
-  program->operating_on_default_kernels = 0;
 
   errcode = CL_SUCCESS;
   goto FINISH;
 
 ERROR:
-  program->kernels = 0;
-  for(i = 0; i < program->num_devices; i++)
-  {
-    if (program->source)
-      {
-        POCL_MEM_FREE (program->binaries[i]);
-        program->binary_sizes[i] = 0;
-      }
-  }
-  if (program->num_kernels && program->kernel_names)
-    {
-      for (i=0; i < program->num_kernels; i++)
-        POCL_MEM_FREE(program->kernel_names[i]);
-      POCL_MEM_FREE(program->kernel_names);
-    }
-  if (program->default_kernels)
-    {
-      program->operating_on_default_kernels = 1;
-      for (i=0; i < program->num_kernels; i++)
-        if (program->default_kernels[i])
-          POname(clReleaseKernel)(program->default_kernels[i]);
-      program->operating_on_default_kernels = 0;
-      POCL_MEM_FREE(program->default_kernels);
-    }
+  free_meta (program);
 
+  program->kernels = NULL;
+
+  for (device_i = 0; device_i < program->num_devices; device_i++)
+    {
+      if (program->source)
+        {
+          POCL_MEM_FREE (program->binaries[device_i]);
+          program->binary_sizes[device_i] = 0;
+        }
+    }
 
 ERROR_CLEAN_OPTIONS:
+  if (temp_options != options)
+    free (temp_options);
+
   program->build_status = CL_BUILD_ERROR;
 
 FINISH:
-  POCL_UNLOCK_OBJ(program);
+  POCL_UNLOCK_OBJ (program);
   POCL_MEM_FREE (unique_devlist);
 
 PFN_NOTIFY:
