@@ -91,14 +91,14 @@ void pocl_unlock_events_inorder (cl_event ev1, cl_event ev2);
 
 /* Function for creating events */
 cl_int pocl_create_event (cl_event *event, cl_command_queue command_queue,
-                          cl_command_type command_type, int num_buffers,
+                          cl_command_type command_type, size_t num_buffers,
                           const cl_mem* buffers, cl_context context);
 
 cl_int pocl_create_command (_cl_command_node **cmd,
                             cl_command_queue command_queue,
                             cl_command_type command_type, cl_event *event,
                             cl_int num_events, const cl_event *wait_list,
-                            int num_buffers, const cl_mem *buffers);
+                            size_t num_buffers, const cl_mem *buffers);
 
 
 void pocl_command_enqueue (cl_command_queue command_queue,
@@ -143,8 +143,9 @@ pocl_command_is_ready(cl_event event)
   return event->wait_list == NULL;
 }
 
-int
-pocl_update_command_queue (cl_event event);
+typedef void (*empty_queue_callback) (cl_command_queue cq);
+
+void pocl_update_command_queue (cl_event event, empty_queue_callback p);
 
 void pocl_cl_mem_inherit_flags (cl_mem mem, cl_mem from_buffer,
                                 cl_mem_flags flags);
@@ -173,6 +174,11 @@ int pocl_check_event_wait_list(cl_command_queue     command_queue,
                                cl_uint              num_events_in_wait_list,
                                const cl_event *     event_wait_list);
 
+void pocl_abort_on_pthread_error (int status, unsigned line, const char *func);
+
+#define PTHREAD_CHECK(code)                                                   \
+  pocl_abort_on_pthread_error ((code), __LINE__, __FUNCTION__);
+
 const char*
 pocl_status_to_str (int status);
 
@@ -200,18 +206,21 @@ float half_to_float (uint16_t value);
  * code can be shared.
  */
 
-#define POCL_RETURN_GETINFO_INNER(__SIZE__, MEMASSIGN)                  \
-  do                                                                    \
-    {                                                                   \
-      if (param_value)                                                  \
-        {                                                               \
-          if (param_value_size < __SIZE__) return CL_INVALID_VALUE;     \
-          MEMASSIGN;                                                    \
-        }                                                               \
-      if (param_value_size_ret)                                         \
-        *param_value_size_ret = __SIZE__;                               \
-      return CL_SUCCESS;                                                \
-    }                                                                   \
+#define POCL_RETURN_GETINFO_INNER(__SIZE__, MEMASSIGN)                        \
+  do                                                                          \
+    {                                                                         \
+      if (param_value)                                                        \
+        {                                                                     \
+          POCL_RETURN_ERROR_ON (                                              \
+              (param_value_size < __SIZE__), CL_INVALID_VALUE,                \
+              "param_value_size (%zu) smaller than actual size (%zu)\n",      \
+              param_value_size, __SIZE__);                                    \
+          MEMASSIGN;                                                          \
+        }                                                                     \
+      if (param_value_size_ret)                                               \
+        *param_value_size_ret = __SIZE__;                                     \
+      return CL_SUCCESS;                                                      \
+    }                                                                         \
   while (0)
 
 #define POCL_RETURN_GETINFO_SIZE(__SIZE__, __POINTER__)                 \
